
#include "utils.h"

#include <QCoreApplication>
#include <QDir>
#include <QFileInfo>
#include <QFont>
#include <QMessageBox>
#include <QPushButton>

#include "ui_common/ui_common.h"

#include "default_config.h"


static QIcon loadIcon(const QString &name)
{
    QIcon icon;

    static const auto iconsDir =
        QDir::fromNativeSeparators(uiGetDir(UiDirData)) + "/icons";

    static const auto sizes = QDir(iconsDir).entryList(QDir::Dirs);
    for (const auto& size : sizes) {
        const QFileInfo fileInfo(
            QString(iconsDir + "/%1/%2.png").arg(size, name));
        if (fileInfo.exists())
            icon.addFile(fileInfo.filePath());
    }

    return icon;
}


QIcon getIcon(const QString &name)
{
    #if defined(Q_OS_UNIX) && !defined(Q_OS_DARWIN)

    if (QIcon::hasThemeIcon(name))
        return QIcon::fromTheme(name);

    #endif

    return loadIcon(name);
}


QPixmap getPixmap(
    const QIcon& icon, int size, QIcon::Mode mode, QIcon::State state)
{
    auto pixmap = icon.pixmap(size, mode, state);

    #if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)

    if (pixmap.height() > size)
        pixmap = pixmap.scaledToHeight(
            size, Qt::SmoothTransformation);

    #endif

    return pixmap;
}


bool confirmation(
    QWidget* parent,
    const QString& question,
    const QString& cancelText,
    const QString& okText)
{
    QMessageBox confirmBox(parent);

    confirmBox.setWindowTitle(uiAppName);
    confirmBox.setText(question);
    confirmBox.setIcon(QMessageBox::Question);

    auto* cancelButton = confirmBox.addButton(
        cancelText, QMessageBox::RejectRole);
    confirmBox.setDefaultButton(cancelButton);
    auto* okButton = confirmBox.addButton(
        okText, QMessageBox::AcceptRole);

    confirmBox.exec();

    return confirmBox.clickedButton() == okButton;
}


void setMonospace(QFont& font)
{
    #ifdef Q_OS_WIN
    font.setFamily("Consolas");
    #else
    font.setFamily("monospace");
    #endif
    font.setStyleHint(QFont::Monospace);
}
